@@ -18,6 +18,7 @@ mod probe;
 mod state;
 mod trace;
 mod tui;
+mod update;
 
 use cli::Args;
 use config::Config;
@@ -35,29 +36,37 @@ use state::{Session, Target, run_ratelimit_worker};
 use trace::engine::ProbeEngine;
 use trace::pending::new_pending_map;
 use trace::receiver::{ReceiverConfig, SessionMap, spawn_receiver};
-use tui::app::{ResolveInfo, run_tui};
+use tui::app::{ReplayState, ResolveInfo, run_tui};
 
 #[tokio::main]
 async fn main() -> Result<()> {
     let args = Args::parse();
 
-    // Handle shell completion generation (before validation, doesn't need targets)
+    // Handle shell completion generation (instant, no update check needed)
     if let Some(ref shell) = args.completions {
         generate_completions(shell);
         return Ok(());
     }
 
+    // Handle replay mode (quick viewing operation, no update check)
+    if let Some(ref replay_path) = args.replay {
+        return run_replay_mode(&args, replay_path).await;
+    }
+
+    // Spawn background update check after early exits
+    // Uses channel for non-blocking result retrieval at exit
+    let (update_tx, update_rx) = std::sync::mpsc::channel();
+    std::thread::spawn(move || {
+        let result = update::check_for_update();
+        let _ = update_tx.send(result); // Ignore if receiver dropped
+    });
+
     // Validate arguments
     if let Err(e) = args.validate() {
         eprintln!("Error: {}", e);
         std::process::exit(1);
     }
 
-    // Handle replay mode (doesn't need permissions or target resolution)
-    if let Some(ref replay_path) = args.replay {
-        return run_replay_mode(&args, replay_path).await;
-    }
-
     // Check permissions early
     if let Err(e) = check_permissions() {
         eprintln!("{}", e);
@@ -224,7 +233,7 @@ async fn main() -> Result<()> {
     }
 
     // Run in appropriate mode
-    if args.is_batch_mode() {
+    let result = if args.is_batch_mode() {
         run_batch_mode(
             args,
             sessions,
@@ -247,7 +256,8 @@ async fn main() -> Result<()> {
         )
         .await
     } else {
-        run_interactive_mode(
+        // Interactive (TUI) mode - pass update_rx for in-app notification
+        return run_interactive_mode(
             args,
             sessions,
             targets,
@@ -255,9 +265,20 @@ async fn main() -> Result<()> {
             cancel,
             interface_info,
             resolve_info,
+            update_rx,
         )
-        .await
+        .await;
+    };
+
+    // Check for update notification (only for non-interactive mode)
+    // Use short timeout so we don't delay exit if check is slow
+    if is_terminal::is_terminal(std::io::stderr())
+        && let Ok(Some(new_version)) = update_rx.recv_timeout(Duration::from_millis(100))
+    {
+        update::print_update_notice(&new_version);
     }
+
+    result
 }
 
 /// Load a session from a JSON file
@@ -294,8 +315,24 @@ async fn run_replay_mode(args: &Args, replay_path: &str) -> Result<()> {
         // Default to report for replay without TUI
         generate_report(&session, std::io::stdout())?;
     } else {
-        // Show in TUI (read-only)
-        let state = Arc::new(RwLock::new(session));
+        // Check for animated replay mode
+        let (session_to_display, replay_state) = if args.animate {
+            if session.events.is_empty() {
+                eprintln!("Note: No event timeline in session file; showing final state.");
+                (session, None)
+            } else {
+                // Create fresh session with same config but no data
+                let events = session.events.clone();
+                let fresh_session = Session::new(session.target.clone(), session.config.clone());
+                let replay = ReplayState::new(events, args.speed);
+                (fresh_session, Some(replay))
+            }
+        } else {
+            (session, None)
+        };
+
+        // Show in TUI
+        let state = Arc::new(RwLock::new(session_to_display));
         let cancel = CancellationToken::new();
 
         // Create SessionMap with single session
@@ -322,7 +359,17 @@ async fn run_replay_mode(args: &Args, replay_path: &str) -> Result<()> {
             cancel_clone.cancel();
         });
 
-        let final_prefs = run_tui(sessions, targets, cancel, prefs, None, None).await?;
+        let final_prefs = run_tui(
+            sessions,
+            targets,
+            cancel,
+            prefs,
+            None,
+            None,
+            None,
+            replay_state,
+        )
+        .await?;
 
         // Save preferences (best effort, don't fail on save error)
         let _ = final_prefs.save();
@@ -389,17 +436,17 @@ fn resolve_targets(
         }
     }
 
-    // Filter by family: prefer IPv4, fall back to IPv6 if no IPv4
-    let has_ipv4 = order.iter().any(|ip| ip.is_ipv4());
-    let has_ipv6 = order.iter().any(|ip| ip.is_ipv6());
+    if order.is_empty() {
+        anyhow::bail!("No addresses found for hostnames");
+    }
 
     let use_ipv6 = if force_ipv6 {
         true
     } else if force_ipv4 {
         false
     } else {
-        // Prefer IPv4, fall back to IPv6
-        !has_ipv4 && has_ipv6
+        // Use IPv6, if first resolved address is IPv6
+        order[0].is_ipv6()
     };
 
     let mut targets = Vec::new();
@@ -471,14 +518,10 @@ fn resolve_target(target: &str, force_ipv4: bool, force_ipv6: bool) -> Result<Ip
         );
     }
 
-    // Prefer IPv4 by default if no preference
-    if !force_ipv6 && let Some(ipv4) = filtered.iter().find(|ip| ip.is_ipv4()) {
-        return Ok(*ipv4);
-    }
-
     Ok(filtered[0])
 }
 
+#[allow(clippy::too_many_arguments)]
 async fn run_interactive_mode(
     args: Args,
     sessions: SessionMap,
@@ -487,6 +530,7 @@ async fn run_interactive_mode(
     cancel: CancellationToken,
     interface: Option<InterfaceInfo>,
     resolve_info: Option<ResolveInfo>,
+    update_rx: std::sync::mpsc::Receiver<Option<String>>,
 ) -> Result<()> {
     // Shared pending map for probe correlation (engine writes, receiver reads)
     let pending = new_pending_map();
@@ -622,6 +666,13 @@ async fn run_interactive_mode(
     // Spawn rate limit detection worker (always enabled, lightweight analysis)
     let ratelimit_handle = tokio::spawn(run_ratelimit_worker(sessions.clone(), cancel.clone()));
 
+    // Check for update result - background check runs during target resolution
+    // Use short timeout since check should already be complete; don't delay startup
+    let update_available = update_rx
+        .recv_timeout(Duration::from_secs(1))
+        .ok()
+        .flatten();
+
     // Run TUI (with target list for cycling)
     let final_prefs = run_tui(
         sessions.clone(),
@@ -630,6 +681,8 @@ async fn run_interactive_mode(
         prefs,
         resolve_info,
         ix_lookup.clone(),
+        update_available,
+        None, // replay_state (live mode)
     )
     .await?;
 
