@@ -10,7 +10,9 @@ use crate::probe::{
     InterfaceInfo, create_recv_socket_with_interface, get_identifier, parse_icmp_response,
     recv_icmp_with_ttl,
 };
-use crate::state::{IcmpResponseType, MplsLabel, PmtudPhase, ProbeId, Session};
+use crate::state::{
+    IcmpResponseType, MplsLabel, PmtudPhase, ProbeEvent, ProbeId, ProbeOutcome, Session,
+};
 use crate::trace::pending::PendingMap;
 
 /// Map of target IP to session, shared across multiple engines and the receiver
@@ -110,7 +112,7 @@ impl Receiver {
         // Set non-blocking with short timeout for polling
         socket.set_read_timeout(Some(Duration::from_millis(100)))?;
 
-        let mut buffer = [0u8; 1500];
+        let mut buffer = [0u8; 9216];
 
         loop {
             // Check cancellation
@@ -227,6 +229,24 @@ impl Receiver {
                                 });
                             } else {
                                 // Late packet arrival - response came after timeout
+                                // Record as LateReply for replay accuracy
+                                if let Some(target) = parsed.original_dest {
+                                    let sessions = self.sessions.read();
+                                    if let Some(session_lock) = sessions.get(&target) {
+                                        let mut state = session_lock.write();
+                                        let offset_ms = state.offset_ms();
+                                        state.record_event(ProbeEvent {
+                                            offset_ms,
+                                            ttl: parsed.probe_id.ttl,
+                                            seq: parsed.probe_id.seq,
+                                            flow_id,
+                                            outcome: ProbeOutcome::LateReply {
+                                                addr: parsed.responder,
+                                                rtt_us: 0, // RTT unknown for late arrivals
+                                            },
+                                        });
+                                    }
+                                }
                                 #[cfg(debug_assertions)]
                                 eprintln!(
                                     "Late response: TTL {} seq {} from {} (already timed out)",
@@ -275,10 +295,15 @@ impl Receiver {
                         let mut state = session.write();
                         let is_pmtud_probe = resp.packet_size.is_some();
 
+                        // Increment sent count here (not in engine) so all stats update atomically
+                        state.total_sent += 1;
+
                         // Only record hop stats for normal probes, not PMTUD probes
                         // PMTUD probes are for MTU discovery, not traceroute measurements
                         if !is_pmtud_probe {
                             if let Some(hop) = state.hop_mut(resp.probe_id.ttl) {
+                                hop.record_sent();
+                                hop.record_flow_sent(resp.flow_id);
                                 // Record aggregate stats with optional flap detection
                                 // Only detect flaps in single-flow mode (multi-flow expects path changes)
                                 if self.config.num_flows == 1 {
@@ -317,6 +342,19 @@ impl Receiver {
                                     hop.record_ttl_manip_check(quoted);
                                 }
                             }
+
+                            // Record event for animated replay (monotonic timing)
+                            let offset_ms = state.offset_ms();
+                            state.record_event(ProbeEvent {
+                                offset_ms,
+                                ttl: resp.probe_id.ttl,
+                                seq: resp.probe_id.seq,
+                                flow_id: resp.flow_id,
+                                outcome: ProbeOutcome::Reply {
+                                    addr: resp.responder,
+                                    rtt_us: resp.rtt.as_micros() as u64,
+                                },
+                            });
                         }
 
                         // Check if we reached the destination
@@ -325,7 +363,7 @@ impl Receiver {
                         {
                             state.complete = true;
                             let ttl = resp.probe_id.ttl;
-                            if state.dest_ttl.is_none() || ttl < state.dest_ttl.unwrap() {
+                            if state.dest_ttl.is_none_or(|d| ttl < d) {
                                 state.dest_ttl = Some(ttl);
                             }
                         }
@@ -377,12 +415,27 @@ impl Receiver {
                         if let Some(session) = sessions.get(target) {
                             let mut state = session.write();
 
+                            // Increment sent count here (not in engine) so all stats update atomically
+                            state.total_sent += 1;
+
                             // Only record hop timeouts for normal probes, not PMTUD probes
                             if !is_pmtud_probe {
                                 if let Some(hop) = state.hop_mut(probe_id.ttl) {
+                                    hop.record_sent();
+                                    hop.record_flow_sent(probe.flow_id);
                                     hop.record_timeout();
                                     hop.record_flow_timeout(probe.flow_id);
                                 }
+
+                                // Record event for animated replay (monotonic timing)
+                                let offset_ms = state.offset_ms();
+                                state.record_event(ProbeEvent {
+                                    offset_ms,
+                                    ttl: probe_id.ttl,
+                                    seq: probe_id.seq,
+                                    flow_id: probe.flow_id,
+                                    outcome: ProbeOutcome::Timeout,
+                                });
                             }
 
                             // PMTUD: Record failure for timed out PMTUD probes
